/*
  Copyright (C) 2011 Stefan Stäglich

  This file is part of pung.

  Pung is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  Pung is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with pung.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <gtest/gtest.h>
#include "./Paddel.h"
#include "./Pong.h"

// _____________________________________________________________________________
TEST(PaddelTest, constructor)
{
  Pong pong;
  Paddel* paddel = new Paddel(&pong, 7, SDLK_w, SDLK_d);
  ASSERT_EQ(0, paddel->_xDiff);
  ASSERT_EQ(0, paddel->_yDiff);
  ASSERT_TRUE(SDLK_w == paddel->_moveUpKey);
  ASSERT_TRUE(SDLK_d == paddel->_moveDownKey);
  ASSERT_EQ(0, paddel->_score);
  ASSERT_EQ(7, paddel->_xPos);

  delete paddel;
}

// _____________________________________________________________________________
TEST(PaddelTest, pushBall)
{
  Pong pong;

  // Case 1: Ball in field
  pong._balls[0]->_xDiff = 7;
  pong._leftPaddel->pushBall(0);
  ASSERT_EQ(7, pong._balls[0]->_xDiff);

  // Case 2: Ball leftpaddel up
  pong._balls[0]->_xDiff = -3;
  pong._balls[0]->_xPos = pong._leftPaddel->_xPos + pong._leftPaddel->_xSize;
  pong._balls[0]->_yPos = pong._leftPaddel->_yPos;
  pong._leftPaddel->pushBall(0);
  ASSERT_EQ(3, pong._balls[0]->_xDiff);

  // Case 3: Ball leftpaddel down
  pong._balls[0]->_xDiff = -3;
  pong._balls[0]->_xPos = pong._leftPaddel->_xPos + pong._leftPaddel->_xSize;
  pong._balls[0]->_yPos = pong._leftPaddel->_yPos + pong._leftPaddel->_ySize;
  pong._leftPaddel->pushBall(0);
  ASSERT_EQ(3, pong._balls[0]->_xDiff);

  // Case 4: Ball rightpaddel up
  pong._balls[0]->_xDiff = -3;
  pong._balls[0]->_xPos = pong._leftPaddel->_xPos + pong._leftPaddel->_xSize;
  pong._balls[0]->_yPos = pong._leftPaddel->_yPos;
  pong._leftPaddel->pushBall(0);
  ASSERT_EQ(3, pong._balls[0]->_xDiff);

  // Case 5: Ball rightpaddel down
  pong._balls[0]->_xDiff = -3;
  pong._balls[0]->_xPos = pong._leftPaddel->_xPos + pong._leftPaddel->_xSize;
  pong._balls[0]->_yPos = pong._leftPaddel->_yPos + pong._leftPaddel->_ySize;
  pong._leftPaddel->pushBall(0);
  ASSERT_EQ(3, pong._balls[0]->_xDiff);
}

// _____________________________________________________________________________
TEST(PaddelTest, move)
{
  Pong pong;
  float oldYPos = pong._leftPaddel->_yPos;
  pong._leftPaddel->_ySize = 10;

  pong._leftPaddel->move();
  ASSERT_EQ(oldYPos, pong._leftPaddel->_yPos);

  pong._leftPaddel->_yDiff = -5;
  oldYPos -= 5;
  pong._leftPaddel->move();
  ASSERT_EQ(oldYPos, pong._leftPaddel->_yPos);

  pong._leftPaddel->_yPos = 0;
  pong._leftPaddel->move();
  ASSERT_EQ(0, pong._leftPaddel->_yDiff);
  ASSERT_EQ(1, pong._leftPaddel->_yPos);

  pong._leftPaddel->_yPos = pong._hres;
  pong._leftPaddel->move();
  ASSERT_EQ(0, pong._leftPaddel->_yDiff);
  ASSERT_EQ(pong._hres - 11, pong._leftPaddel->_yPos);
}

// _____________________________________________________________________________
TEST(PaddelTest, reset)
{
  Pong pong;
  float oldYPos = pong._leftPaddel->_yPos;
  pong._leftPaddel->_yPos = 2;

  pong._leftPaddel->reset();
  ASSERT_EQ(oldYPos, pong._leftPaddel->_yPos);
}
